unit DynArray;

interface

uses
  Objects;

{$R-}  { Desactivation de la detection de domaine }

type
  PDynamicArray = ^TDynamicArray;
  TDynamicArray = object(TObject)
  private
    FSize: Word;
    FCellSize: Word;
    FValues: Pointer;
  public
    function At(APos: Word): Pointer;
    function SetLength(ASize: Word): Boolean;
    function Length: Word;
    constructor Init(ACellSize: Word);
    destructor Done; virtual;
  end;


implementation


function TDynamicArray.At(APos: Word): Pointer;
begin
  At := nil;
  if APos >= FSize then Exit;                                 { Verification des bornes }
  At := Ptr(Seg(FValues^), Ofs(FValues^) + APos * FCellSize); { Renvoie un pointeur vers la valeur }
end;



function TDynamicArray.SetLength(ASize: Word): Boolean;
var
  P: Pointer;
begin
  SetLength := False;
  if ASize = 0 then                            { Suppression du tableau }
  begin
    if Assigned(FValues) then                  { Si le tableau est alloue... }
    begin
      FreeMem(FValues, FSize * FCellSize);     { On le libere }
      FSize := 0;                              { Modification des parametres }
      FValues := nil;
      
      SetLength := True;
    end;
  end
  else                                         { Redimensionnement }
  begin
    GetMem(P, ASize * FCellSize);              { Allocation du nouveau tableau }
    if not Assigned(P) then Exit;
    
    if FSize <> 0 then                         { S'il y a deja eu allocation... }
    begin
      if ASize < FSize then
        Move(FValues^, P^, ASize * FCellSize)  { Copie des elements precedents }
      else
        Move(FValues^, P^, FSize * FCellSize);
      
      FreeMem(FValues, FSize * FCellSize);     { Liberation de l'ancien tableau }
    end;
    
    FSize := ASize;                            { Modification des parametres }
    FValues := P;
    
    SetLength := True;
  end;
end;



function TDynamicArray.Length: Word;
begin
  Length := FSize;
end;



constructor TDynamicArray.Init(ACellSize: Word);
begin
  if not inherited Init then Fail;   { Appel de l'ancetre }
  if ACellSize < 1 then              { Sauvegarde des parametres }
    Fail
  else
    FCellSize := ACellSize;
end;



destructor TDynamicArray.Done;
begin
  if Assigned(FValues) then    { On libere le tableau s'il est encore alloue }
    SetLength(0);
  inherited Done;              { Appel de l'ancetre }
end;

end.